from infi.clickhouse_orm import fields, engines

from .fields import DateField, DateTimeField, BoolStringField
from .db import CLUSTER_NAME, Model, DistributedModel


class DungeonBattle(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    level_id = fields.StringField()
    is_win = BoolStringField()
    monster_id = fields.StringField()
    monster_souls = fields.StringField()
    monster_rarity = fields.StringField()
    league = fields.UInt64Field(default=32)

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class DungeonBattleDist(DungeonBattle, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class Pits(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    action = fields.StringField()
    league = fields.UInt64Field(default=32)
    pit_id = fields.UInt64Field()
    pit_level = fields.StringField()
    start_datetime = fields.StringField()
    current_hp = fields.StringField()
    bought_pit_keys = fields.UInt64Field()
    spent_gems = fields.UInt64Field()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class PitsDist(Pits, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class PvpStats(Model):
    day = DateField()
    created_on = DateTimeField()
    finished_on = DateTimeField()
    attacker_id = fields.StringField()
    defender_id = fields.StringField()
    match_type = fields.StringField()
    match_id = fields.UInt64Field()
    fame = fields.UInt64Field()
    league = fields.UInt64Field(default=32)
    strike = fields.UInt64Field()
    is_win = BoolStringField()
    attacker_win_fame_change = fields.StringField()
    attacker_lose_fame_change = fields.StringField()
    attacker_start_fame = fields.StringField()
    attacker_start_base_collection_armypower = fields.StringField()
    attacker_start_collection_armypower = fields.StringField()
    is_first = fields.StringField()
    survival_strike = fields.StringField()
    enemy_warlord_hp = fields.StringField()
    attacker_survival_base_collection_armypower = fields.Float64Field()
    attacker_survival_collection_armypower = fields.Float64Field() # MP Default 0.
    defender_survival_base_collection_armypower = fields.Float64Field() # MP Default 0.
    defender_survival_collection_armypower = fields.Float64Field() # MP Default 0.
    attacker_base_armypower = fields.Float64Field()
    attacker_cur_armypower = fields.Float64Field()
    attacker_max_armypower = fields.Float64Field()
    defender_base_armypower = fields.Float64Field()
    defender_cur_armypower = fields.Float64Field()
    defender_max_armypower = fields.Float64Field()

    engine = engines.MergeTree('day', ('attacker_id', 'day'))


class PvpStatsDist(PvpStats, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class PvpWarlordSkin(Model):
    day = DateField()
    created_on = DateTimeField()
    attacker_id = fields.StringField()
    opponent_id = fields.StringField()
    league = fields.StringField(default='32')
    match_id = fields.StringField()
    skin_id = fields.StringField()

    engine = engines.MergeTree('day', ('attacker_id', 'day'))


class PvpWarlordSkinDist(PvpWarlordSkin, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class TavernBattle(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    level_id = fields.StringField()
    is_win = BoolStringField()
    tavern_id = fields.StringField()
    warlord_id = fields.StringField()
    league = fields.UInt64Field(default=32)

    engine = engines.MergeTree('day', ('day', 'profile_id'))


class TavernBattleDist(TavernBattle, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)



from infi.clickhouse_orm import fields, engines

from .fields import DateField, DateTimeField, ListStringField, BoolStringField
from .db import CLUSTER_NAME, Model, DistributedModel


class EventLot(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    league = fields.UInt64Field(default=32)
    event_id = fields.UInt64Field()
    lot_string_id = fields.StringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class EventLotDist(EventLot, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class EventShopLot(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    league = fields.UInt64Field(default=32)
    event_id = fields.UInt64Field()
    lot_id = fields.UInt64Field()
    price = fields.Float64Field()
    platform_id = fields.StringField() # queue only
    env = fields.StringField() # queue only

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class EventShopLotDist(EventShopLot, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class EventTotalPurchases(Model):
    event_id = fields.UInt64Field() # in AP only
    quest_id = fields.UInt64Field()
    price = fields.Float64Field()

    engine = engines.Log()


class ItemShown(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    item_types = ListStringField()
    item_ids = ListStringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class ItemShownDist(ItemShown, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class LotApplied(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    item_id = fields.StringField()
    price = fields.Float64Field()
    price_res_type = fields.StringField()
    action_id = fields.StringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class LotAppliedDist(LotApplied, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class Purchases(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    league = fields.UInt64Field(default=32)
    lot_id = fields.StringField()
    lot_string_id = fields.StringField()
    environment = fields.StringField()
    transaction_id = fields.StringField()
    is_qa_purchase = BoolStringField()
    currency_code = fields.StringField()
    state = fields.StringField()
    price = fields.Float64Field()
    price_usd = fields.Float64Field()
    rank = fields.UInt64Field()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class PurchasesDist(Purchases, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


# class QuestLotId(Model):
#     event_id = fields.UInt64Field()
#     lot_id = fields.UInt64Field()
#     shop_lot = fields.UInt32Field()
#
#     engine = engines.Log()


# class QuestShopLots(Model):
#     quest_id = fields.UInt64Field()
#     chapter_id = fields.UInt64Field()
#     shop_lot = fields.UInt32Field()
#
#     engine = engines.Log()


class SkinPurchase(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    platform = fields.StringField()
    league = fields.UInt64Field(default=32)
    skin_id = fields.StringField()
    price = fields.StringField()
    price_resource = fields.StringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class SkinPurchaseDist(SkinPurchase, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)



import ast
import json

from infi.clickhouse_orm import fields
from infi.clickhouse_orm.utils import escape
from six import text_type, string_types


class BoolField(fields.Field):
    class_default = False
    db_type = 'UInt8'

    def to_python(self, value, timezone_in_use):
        if isinstance(value, string_types):
            value = value.lower()
        if value in (1, '1', True, 'true'):
            return True
        elif value in (0, '0', False, 'false'):
            return False
        else:
            raise ValueError('Invalid value for %s - %r' % (self.__class__.__name__, value))

    def to_db_string(self, value, quote=True):
        return text_type(self.to_kafka(value))

    def to_kafka(self, value):
        return int(value)


class BoolStringField(BoolField):
    class_default = False
    db_type = 'String'

    def to_kafka(self, value):
        return str(value).lower()


class DateField(fields.DateField):
    def to_kafka(self, value):
        return self.to_db_string(value, quote=False)


class DateTimeField(fields.DateTimeField):
    def to_kafka(self, value):
        return self.to_db_string(value, quote=False)


class JsonStringField(fields.StringField):
    class_default = {}

    def to_python(self, value, timezone_in_use):
        if isinstance(value, string_types):
            return json.loads(value)
        elif isinstance(value, (dict, list)):
            return value
        else:
            raise ValueError('Invalid value for %s - %r' % (self.__class__.__name__, value))

    def to_db_string(self, value, quote=True):
        return escape(self.to_kafka(value), quote)

    def to_kafka(self, value):
        return json.dumps(value)


class ListStringField(fields.StringField):
    class_default = []

    def to_python(self, value, timezone_in_use):
        if isinstance(value, string_types):
            return ast.literal_eval(value)
        if isinstance(value, list):
            return value
        else:
            raise ValueError('Invalid value for %s - %r' % (self.__class__.__name__, value))

    def to_kafka(self, value):
        return self.to_db_string(value, quote=False)


from infi.clickhouse_orm import fields, engines

from .fields import DateField, DateTimeField, BoolStringField
from .db import CLUSTER_NAME, Model, DistributedModel


class RaidBoosts(Model):
    day = DateField()
    created_on = DateTimeField()
    raid_id = fields.UInt64Field()
    room_number = fields.UInt64Field()
    room_group_number = fields.UInt64Field()
    start_datetime = DateTimeField()
    end_datetime = DateTimeField()
    is_qa_room = BoolStringField()
    profile_id = fields.StringField()
    attack_boost = BoolStringField()
    spent_resource = fields.UInt64Field()
    times_boosted = fields.UInt64Field()

    engine = engines.MergeTree('day', ('day', 'raid_id', 'room_number'))


class RaidBoostsDist(RaidBoosts, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class RaidElixirSpent(Model):
    day = DateField()
    created_on = DateTimeField()
    raid_id = fields.UInt64Field()
    room_number = fields.UInt64Field()
    room_group_number = fields.UInt64Field()
    profile_id = fields.StringField()
    action = fields.StringField()
    elixir_spent = fields.UInt64Field()

    engine = engines.MergeTree('day', ('day', 'raid_id', 'room_number'))


class RaidElixirSpentDist(RaidElixirSpent, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class RaidForces(Model):
    day = DateField()
    created_on = DateTimeField()
    raid_id = fields.UInt64Field()
    room_number = fields.UInt64Field()
    room_group_number = fields.UInt64Field()
    start_datetime = DateTimeField()
    end_datetime = DateTimeField()
    is_qa_room = BoolStringField()
    profile_id = fields.StringField()
    attack_force = BoolStringField()
    cell_x = fields.UInt64Field()
    cell_y = fields.UInt64Field()
    force_start_datetime = DateTimeField()
    power = fields.UInt64Field()
    pure_power = fields.UInt64Field()
    arrive_seconds = fields.UInt64Field()
    midway_seconds = fields.UInt64Field()

    engine = engines.MergeTree('day', ('day', 'raid_id', 'room_number'))


class RaidForcesDist(RaidForces, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class RaidRooms(Model):
    day = DateField()
    created_on = DateTimeField()
    raid_id = fields.UInt64Field()
    room_number = fields.UInt64Field()
    room_group_number = fields.UInt64Field()
    start_datetime = DateTimeField()
    end_datetime = DateTimeField()
    is_qa_room = BoolStringField()
    profile_id = fields.StringField()
    start_raid_points = fields.UInt64Field()
    raid_points = fields.UInt64Field()
    static_raid_points = fields.UInt64Field()
    total_raid_points = fields.UInt64Field()
    start_total_raid_points = fields.UInt64Field()
    summary_guild_power = fields.UInt64Field()

    engine = engines.MergeTree('day', ('day', 'raid_id', 'room_number'))


class RaidRoomsDist(RaidRooms, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class RaidSpeedups(Model):
    day = DateField()
    created_on = DateTimeField()
    raid_id = fields.UInt64Field()
    room_number = fields.UInt64Field()
    room_group_number = fields.UInt64Field()
    start_datetime = DateTimeField()
    end_datetime = DateTimeField()
    is_qa_room = BoolStringField()
    profile_id = fields.StringField()
    is_forward = BoolStringField()
    spent_resource = fields.UInt64Field()
    cell_x = fields.UInt64Field()
    cell_y = fields.UInt64Field()
    force_start_datetime = fields.UInt64Field()

    engine = engines.MergeTree('day', ('day', 'raid_id', 'room_number'))


class RaidSpeedupsDist(RaidSpeedups, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class RaidTroopsHiding(Model):
    day = DateField()
    created_on = DateTimeField()
    raid_id = fields.UInt64Field()
    room_number = fields.UInt64Field()
    room_group_number = fields.UInt64Field()
    start_datetime = DateTimeField()
    end_datetime = DateTimeField()
    is_qa_room = BoolStringField()
    profile_id = fields.StringField()
    hidden = BoolStringField()
    spent_resource = fields.UInt64Field()
    cell_x = fields.UInt64Field()
    cell_y = fields.UInt64Field()
    force_start_datetime = DateTimeField()

    engine = engines.MergeTree('day', ('day', 'raid_id', 'room_number'))


class RaidTroopsHidingDist(RaidTroopsHiding, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)




from infi.clickhouse_orm import fields, engines

from .fields import DateField, DateTimeField
from .db import CLUSTER_NAME, Model, DistributedModel


class EventQuestCompletion(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    league = fields.UInt64Field(default=32)
    event_id = fields.UInt64Field()
    quest_id = fields.UInt64Field()
    stars = fields.UInt64Field()
    chapter = fields.UInt64Field()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class EventQuestCompletionDist(EventQuestCompletion, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class EventStart(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    league = fields.UInt64Field(default=32)
    event_id = fields.UInt64Field()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class EventStartDist(EventStart, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class MiniEvents(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    action = fields.StringField()
    platform = fields.StringField()
    mini_event_id = fields.UInt64Field()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class MiniEventsDist(MiniEvents, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class QuestChapters(Model):
    day = DateField()
    event_id = fields.UInt64Field()
    chapter_number = fields.UInt32Field()
    chapter_id = fields.UInt32Field()

    engine = engines.MergeTree('day', ('chapter_id', 'day'))


class QuestChaptersDist(QuestChapters, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class QuestIds(Model):
    day = DateField()
    event_id = fields.UInt64Field()
    quest_id = fields.UInt64Field()
    chapter_id = fields.UInt32Field()

    engine = engines.MergeTree('day', ('quest_id', 'day'))

class QuestIdsDist(QuestIds, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)



from infi.clickhouse_orm import fields, engines

from .fields import DateField, DateTimeField, JsonStringField, BoolStringField
from .db import CLUSTER_NAME, Model, DistributedModel


class RewardApplied(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    reward_id = fields.StringField()
    source = fields.StringField()
    reward_type = fields.StringField()
    count = fields.UInt64Field()
    friend = fields.StringField()
    extra = JsonStringField()
    autoclaim = BoolStringField()
    reward_created_on = fields.StringField()
    accepted = BoolStringField()
    id = fields.StringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class RewardAppliedDist(RewardApplied, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class Rewards(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    action = fields.StringField()
    custom_id = fields.StringField()
    reward_type = fields.StringField()
    reward_id = fields.StringField()
    source = fields.StringField()
    count = fields.UInt64Field()
    friend = fields.StringField()
    extra = JsonStringField()
    autoclaim = BoolStringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class RewardsDist(Rewards, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)





from infi.clickhouse_orm import fields, engines

from .fields import DateField, DateTimeField, JsonStringField, BoolStringField
from .db import CLUSTER_NAME, Model, DistributedModel


class ArmyCollectionLog(Model):
    day = DateField()
    profile_id = fields.StringField()
    created_on = DateTimeField()
    actions = JsonStringField()
    difference = JsonStringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class ArmyCollectionLogDist(ArmyCollectionLog, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class CheaterMarks(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    is_paying = BoolStringField()
    action = fields.StringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class CheaterMarksDist(CheaterMarks, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class Registers(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    env = fields.StringField()
    player_country = fields.StringField()
    utm_campaign = fields.StringField()
    utm_medium = fields.StringField()
    utm_source = fields.StringField()
    utm_full = fields.StringField()
    platform_id = fields.StringField()
    advertising_id = fields.StringField() # MP only

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class RegistersDist(Registers, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class ResourcesLog(Model):
    day = DateField()
    profile_id = fields.StringField()
    created_on = DateTimeField()
    actions = JsonStringField()
    difference = JsonStringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class ResourcesLogDist(ResourcesLog, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class Sessions(Model):
    day = DateField()
    event_type = fields.StringField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    session_id = fields.StringField()
    env = fields.StringField()
    device_id = fields.StringField()
    player_country = fields.StringField()
    platform_id = fields.StringField()
    client_ip = fields.StringField()
    login_id = fields.StringField() # AP only
    request_id = fields.StringField() # AP only

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class SessionsDist(Sessions, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)


class TroopsHiding(Model):
    day = DateField()
    created_on = DateTimeField()
    profile_id = fields.StringField()
    guild_id = fields.UInt64Field()
    guild_name = fields.StringField()
    war_id = fields.UInt64Field()
    room_number = fields.UInt64Field()
    spent_gems = fields.UInt64Field()
    cell_x = fields.UInt64Field()
    cell_y = fields.UInt64Field()
    hidden = BoolStringField()

    engine = engines.MergeTree('day', ('profile_id', 'day'))


class TroopsHidingDist(TroopsHiding, DistributedModel):
    engine = engines.Distributed(CLUSTER_NAME)
